var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "path", "hash-sum", "san-sfc-compiler", "./template", "./script", "./style", "../utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const path_1 = __importDefault(require("path"));
    const hash_sum_1 = __importDefault(require("hash-sum"));
    const san_sfc_compiler_1 = require("san-sfc-compiler");
    const template_1 = __importDefault(require("./template"));
    const script_1 = __importDefault(require("./script"));
    const style_1 = __importDefault(require("./style"));
    const utils_1 = require("../utils");
    exports.default = (source, filename, options, sourceRoot) => {
        var _a;
        const descriptor = (0, san_sfc_compiler_1.parseSFC)({
            source,
            filename,
            sourceRoot,
            needMap: true,
        });
        (0, utils_1.setDescriptor)(filename, descriptor);
        const shortFilePath = path_1.default
            .relative(sourceRoot, filename)
            .replace(/^(\.\.[\/\\])+/, '')
            .replace(/\\/g, '/');
        const scopeId = (0, hash_sum_1.default)(shortFilePath);
        const templateImport = (0, template_1.default)(descriptor, scopeId);
        const scriptImport = (0, script_1.default)(descriptor, scopeId);
        const stylesImport = (0, style_1.default)(descriptor, scopeId, options.preprocessStyles);
        let templateType = 'template';
        const anode = (_a = options.templateCompileOptions) === null || _a === void 0 ? void 0 : _a.compileANode;
        if (anode && anode !== 'none') {
            templateType = anode;
        }
        const hasCSSModules = descriptor.styles.some((s) => s.module);
        const injectStyleToData = hasCSSModules
            ? `var origin = typeof script === 'function'
    ? (script.prototype.initData || __noData)
    : (script.initData || __noData);
    function __noData() {
      return {$style: $style};
    }
    function __finalInitData() {
        return Object.assign({}, origin.call(this), {$style: $style});
    }
    if (typeof script === 'function') {
      script.prototype.initData = __finalInitData;
    } else {
      script.initData = __finalInitData;
    }`
            : '';
        const output = [
            scriptImport,
            templateImport,
            stylesImport,
            `script.${templateType} = template`,
            injectStyleToData,
            'export default script',
        ];
        return {
            code: output.join('\n'),
            map: {
                version: 3,
                mappings: '',
                sources: [],
                names: [],
            },
        };
    };
});
