(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const utils_1 = require("../utils");
    function genCSSModulesCode(index, request) {
        const styleVar = `style${index}`;
        let code = `\nimport ${JSON.stringify(request)}` +
            // 运行时
            `\nimport ${styleVar} from ${JSON.stringify(request + '.js')}`;
        code += `\n$style = Object.assign($style, ${styleVar})`;
        return code;
    }
    exports.default = (descriptor, scopeId, preprocessStyles) => {
        let stylesCode = ``;
        // 全局开关
        let hasCSSModules = false;
        if (descriptor.styles.length) {
            descriptor.styles.forEach((style, i) => {
                const src = style.src || descriptor.filename;
                const attrsQuery = (0, utils_1.attrsToQuery)(style.attrs, 'css', preprocessStyles);
                const idQuery = `&id=${scopeId}`;
                const srcQuery = style.src ? `&src` : ``;
                const query = `?san&type=style&index=${i}${srcQuery}${idQuery}`;
                const styleRequest = src + query + attrsQuery;
                if (style.module) {
                    if (!hasCSSModules) {
                        stylesCode += `\nvar $style = {};`;
                        // 防止重复赋值
                        hasCSSModules = true;
                    }
                    stylesCode += genCSSModulesCode(i, styleRequest);
                }
                else {
                    stylesCode += `\nimport ${JSON.stringify(styleRequest)}`;
                }
            });
        }
        return stylesCode;
    };
});
