var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "rollup-pluginutils", "./gencode/entry", "./transform/template", "./transform/style", "./utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const fs_1 = __importDefault(require("fs"));
    const rollup_pluginutils_1 = require("rollup-pluginutils");
    const entry_1 = __importDefault(require("./gencode/entry"));
    const template_1 = __importDefault(require("./transform/template"));
    const style_1 = __importDefault(require("./transform/style"));
    const utils_1 = require("./utils");
    const defaultOptions = {
        include: /\.san$/,
        exclude: [],
    };
    function PluginSan(userOptions = {}) {
        const options = Object.assign(Object.assign({}, defaultOptions), userOptions);
        const filter = (0, rollup_pluginutils_1.createFilter)(options.include, options.exclude);
        return {
            name: 'san',
            resolveId(id, importer) {
                return __awaiter(this, void 0, void 0, function* () {
                    const query = (0, utils_1.parseSanPartRequest)(id);
                    if (query.san) {
                        if (query.src) {
                            const resolved = yield this.resolve(query.filename, importer, {
                                skipSelf: true,
                            });
                            if (resolved) {
                                (0, utils_1.setDescriptor)(resolved.id, (0, utils_1.getDescriptor)(importer));
                                const [, originalQuery] = id.split('?', 2);
                                resolved.id += `?${originalQuery}`;
                                return resolved;
                            }
                        }
                        else if (!filter(query.filename)) {
                            return null;
                        }
                        return id;
                    }
                    return null;
                });
            },
            load(id) {
                const query = (0, utils_1.parseSanPartRequest)(id);
                if (query.san) {
                    if (query.src) {
                        return fs_1.default.readFileSync(query.filename, 'utf-8');
                    }
                    const descriptor = (0, utils_1.getDescriptor)(query.filename);
                    if (descriptor) {
                        const block = query.type === 'template'
                            ? descriptor.template
                            : query.type === 'script'
                                ? descriptor.script
                                : query.type === 'style'
                                    ? descriptor.styles[query.index]
                                    : null;
                        if (block) {
                            return {
                                code: block.content,
                                map: (0, utils_1.normalizeSourceMap)(block.map, id),
                            };
                        }
                    }
                }
                return null;
            },
            transform(code, id) {
                return __awaiter(this, void 0, void 0, function* () {
                    const query = (0, utils_1.parseSanPartRequest)(id);
                    const descriptor = (0, utils_1.getDescriptor)(query.filename);
                    // 生成入口
                    if (!query.san && filter(id)) {
                        return (0, entry_1.default)(code, id, options, process.cwd());
                    }
                    // 不同的块
                    if (query.san) {
                        if (!query.src && !filter(query.filename)) {
                            return null;
                        }
                        if (query.src) {
                            this.addWatchFile(query.filename);
                        }
                        if (query.type === 'template') {
                            const hasScoped = descriptor.styles.some((s) => s.scoped);
                            return (0, template_1.default)(code, id, options, query, hasScoped);
                        }
                        else if (query.type === 'style') {
                            return (0, style_1.default)(code, id, options, query);
                        }
                    }
                    return null;
                });
            },
        };
    }
    exports.default = PluginSan;
    module.exports = PluginSan;
});
