var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "qs"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.getDescriptor = exports.setDescriptor = exports.normalizeSourceMap = exports.parseSanPartRequest = exports.attrsToQuery = void 0;
    const qs_1 = __importDefault(require("qs"));
    function attrsToQuery(attrs, langFallback, forceLangFallback = false) {
        let query = ``;
        for (const name in attrs) {
            const value = attrs[name];
            if (!['id', 'index', 'src', 'type', 'lang'].includes(name)) {
                query += `&${name}${value ? `=${String(value)}` : ``}`;
            }
        }
        if (langFallback || attrs.lang) {
            query +=
                `lang` in attrs
                    ? forceLangFallback
                        ? `&lang.${langFallback}`
                        : `&lang.${attrs.lang}`
                    : `&lang.${langFallback}`;
        }
        return query;
    }
    exports.attrsToQuery = attrsToQuery;
    function parseSanPartRequest(id) {
        const [filename, query] = id.split('?', 2);
        if (!query)
            return { san: false, filename };
        const raw = qs_1.default.parse(query);
        if ('san' in raw) {
            return Object.assign(Object.assign({}, raw), { filename, san: true, index: Number(raw.index), src: 'src' in raw, scoped: 'scoped' in raw });
        }
        return { san: false, filename };
    }
    exports.parseSanPartRequest = parseSanPartRequest;
    function normalizeSourceMap(map, request) {
        if (!map)
            return null;
        if (!request.includes('type=script')) {
            map.file = request;
            map.sources = map.sources || [];
            map.sources[0] = request;
        }
        return Object.assign(Object.assign({}, map), { version: Number(map.version), mappings: typeof map.mappings === 'string' ? map.mappings : '' });
    }
    exports.normalizeSourceMap = normalizeSourceMap;
    // sfc descriptor
    const cache = new Map();
    function setDescriptor(id, entry) {
        cache.set(id, entry);
    }
    exports.setDescriptor = setDescriptor;
    function getDescriptor(id) {
        if (cache.has(id)) {
            return cache.get(id);
        }
        return null;
    }
    exports.getDescriptor = getDescriptor;
});
